@@ -1,5 +1,5 @@
 // FbDesk.cc
-// Copyright (c) 2002 - 2004 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -43,64 +43,11 @@
 using namespace std;
 using namespace FbTk;
 
-template <>
-void Resource<int>::setFromString(const char *str) {
-    sscanf(str, "%d", &*(*this));
-}
-
-template <>
-std::string Resource<int>::getString() {
-    char strval[256];
-    sprintf(strval, "%d", *(*this));
-    return std::string(strval);
-}
-
-template <>
-void Resource<std::string>::setFromString(const char *str) {
-    *(*this) = (str ? str : "");
-}
-
-template <>
-std::string Resource<std::string>::getString() {
-    return (*this)->c_str();
-}
-
-
-
-template <>
-std::string Resource<FbDesk::Icon::TextPlacement>::getString() {
-    switch (*(*this)) {
-    case FbDesk::Icon::TEXTPLACE_LEFT:
-        return "Left";
-    case FbDesk::Icon::TEXTPLACE_RIGHT:
-        return "Right";
-    case FbDesk::Icon::TEXTPLACE_TOP:
-        return "Top";
-    case FbDesk::Icon::TEXTPLACE_BOTTOM:
-        return "Bottom";
-    }
-    return "Bottom";
-
-}
-
-template <>
-void Resource<FbDesk::Icon::TextPlacement>::setFromString(const char *str) {
-    if (strcmp("Left", str) == 0) {
-        *(*this) = FbDesk::Icon::TEXTPLACE_LEFT;
-    } else if (strcmp("Right", str) == 0) {
-        *(*this) = FbDesk::Icon::TEXTPLACE_RIGHT;
-    } else if (strcmp("Top", str) == 0) 
-        *(*this) = FbDesk::Icon::TEXTPLACE_TOP;
-    else  if (strcmp("Bottom", str) == 0)
-        *(*this) = FbDesk::Icon::TEXTPLACE_BOTTOM;
-    else 
-        *(*this) = FbDesk::Icon::TEXTPLACE_BOTTOM;
-}
 
 namespace FbDesk {
 
 FbDesk::FbDesk(const char *config_filename):
-    m_font("fixed", true),
+    m_font("fixed"),
     m_selected_icon(0),
     m_last_icon(0),
     m_config_filename(config_filename ? config_filename : ""),
@@ -112,19 +59,22 @@ FbDesk::FbDesk(const char *config_filename):
     m_inputfield(0, m_menutheme.frameFont(), m_menutheme.frameTextGC().gc(),
                  m_menutheme.frameTexture().color()),
     m_textcolor("black", 0),
-
     m_resmanager(0, false),
+    m_themefile(m_resmanager, "", "session.styleFile", "Session.StyleFile"),
     m_doubleclick_interval(m_resmanager, 200, "fbdesk.doubleClickInterval", "FbDesk.DoubleClickInterval"),
     m_iconfile(m_resmanager, "~/.fluxbox/fbdesk.icons", "fbdesk.iconFile", "FbDesk.IconFile"),
     m_fontname(m_resmanager, "fixed", "fbdesk.font", "FbDesk.Font"),
     m_textcolorname(m_resmanager, "black", "fbdesk.textColor", "FbDesk.TextColor"),
+    m_textbackgroundname(m_resmanager, "white", "fbdesk.textBackground", "FbDesk.TextBackground"),
     m_grid_snap_x(m_resmanager, 5, "fbdesk.snapX", "FbDesk.SnapX"),
     m_grid_snap_y(m_resmanager, 5, "fbdesk.snapY", "FbDesk.SnapY"),
     m_textplacement(m_resmanager, Icon::TEXTPLACE_BOTTOM, "fbdesk.textPlacement", "FbDesk.TextPlacement"),
     m_text_alpha(m_resmanager, 0, "fbdesk.textAlpha", "FbDesk.TextAlpha"),
-    m_icon_alpha(m_resmanager, 255, "fbdesk.iconAlpha", "FbDesk.IconAlpha") {
+    m_icon_alpha(m_resmanager, 255, "fbdesk.iconAlpha", "FbDesk.IconAlpha"),
+    m_lock_positions(m_resmanager, false, "fbdesk.lockPositions", "FbDesk.LockPositions"),
+    m_icon_config(m_menutheme.screenNum()) {
 
-    Icon::setFont(m_font);
+    m_icon_config.setFont(m_font);
     reloadConfig();
     setupIconMenu();
 
@@ -138,37 +88,20 @@ FbDesk::FbDesk(const char *config_filename):
 FbDesk::~FbDesk() {
     // save icon data
     saveIcons(*m_iconfile);
-    
+    // save to default resource file
+    m_resmanager.save( 0 );
+
     // delete all icons
     deleteIcons();
 }
 
-void FbDesk::setAntialias(bool value) {
-    if (font().isAntialias() != value) {
-       font().setAntialias(value);
-       updateIcons();
-    }
-
-    bool reconf_menu = false;
-    if (m_menutheme.titleFont().isAntialias() != value) {
-        m_menutheme.titleFont().setAntialias(value);
-        reconf_menu = true;
-    }
-
-    if (m_menutheme.frameFont().isAntialias() != value) {
-        m_menutheme.frameFont().setAntialias(value);
-        reconf_menu = true;
-    }
-
-    if (reconf_menu)
-        m_iconmenu.reconfigure();
-
-}
 
 void FbDesk::loadTheme() {
-    FbTk::ThemeManager::instance().load(m_themefile.c_str());
-    m_menutheme.frameFont().setAntialias(m_font.isAntialias());
-    m_menutheme.titleFont().setAntialias(m_font.isAntialias());
+    cerr<<"loading style: " << *m_themefile <<endl;
+
+    if (!FbTk::ThemeManager::instance().load(*m_themefile, ""))
+        cerr<<"Failed to load style: "<< *m_themefile<<endl;
+
     // update for new theme
     m_iconmenu.reconfigure();
 }
@@ -177,21 +110,22 @@ void FbDesk::loadTheme() {
    Destroy old icons and reload icons data and recreate icons
 */
 bool FbDesk::loadIcons(const std::string &config_filename) {
-
-
     ifstream iconfile(StringUtil::expandFilename(config_filename).c_str());
-    cerr<<"Loading: ["<<StringUtil::expandFilename(config_filename).c_str()<<"]"<<endl;
 
     if (!m_iconlist.empty()) {
         // delete old icons
         deleteIcons();
     }
-	
+
+    vector<int> iconpos;
+
     while (iconfile && !iconfile.eof()) {
         string line;
         getline(iconfile, line);
-        if (line != "[Desktop Entry]")
+        if (line.compare(0, strlen("[Desktop Entry]"),
+                         "[Desktop Entry]" )) {
             continue;
+        }
 
         int icon_x = 0, icon_y = 0; // position of icon
         string command;       // command for icon
@@ -203,9 +137,7 @@ bool FbDesk::loadIcons(const std::string &config_filename) {
             getline(iconfile, line); // get next line
 
             //remove any whitespace in the begining
-            size_t erase_from = line.find_first_not_of(" \t\n");
-            if (erase_from != 0 && erase_from != string::npos)
-                line.erase(0, erase_from);
+            FbTk::StringUtil::removeFirstWhitespace(line);
 
             // get first position after option
             size_t first_pos = line.find_first_of("=");
@@ -225,6 +157,9 @@ bool FbDesk::loadIcons(const std::string &config_filename) {
                 option_value = static_cast<const char *>(line.c_str() + first_pos + 1);
             }
 
+            FbTk::StringUtil::removeFirstWhitespace(option);
+            FbTk::StringUtil::removeTrailingWhitespace(option);
+
             if (option == "Name") {
                 label = option_value;
             } else if (option == "Exec") {
@@ -246,8 +181,9 @@ bool FbDesk::loadIcons(const std::string &config_filename) {
                 // add to event manager
                 FbTk::EventManager::instance()->add(*this, i->iconWindow());
                 FbTk::EventManager::instance()->add(*this, i->labelWindow());
-                // move it into position
-                i->move(icon_x, icon_y);
+                // save position
+                iconpos.push_back(icon_x);
+                iconpos.push_back(icon_y);
                 // add it to list
                 m_iconlist.push_back(i);
                 // clear options
@@ -262,11 +198,13 @@ bool FbDesk::loadIcons(const std::string &config_filename) {
     // show all icons
     IconList::iterator icon_it = m_iconlist.begin();
     IconList::iterator icon_it_end = m_iconlist.end();
-    for (; icon_it != icon_it_end; ++icon_it)
+    for (int i = 0; icon_it != icon_it_end; ++icon_it, i += 2) {
         (*icon_it)->show();
+        (*icon_it)->move(iconpos[i], iconpos[i+1]);
+    }
 
     // create at lest one icon
-    if (m_iconlist.size() == 0) {
+    if (m_iconlist.empty()) {
         cerr<<"Warning: Empty icon file, creating one icon."<<endl;
         createIcon();
     }
@@ -304,9 +242,6 @@ void FbDesk::buttonReleaseEvent(XButtonEvent &event) {
 
     if (event.button == 3) {
         // show menu
-#ifdef DEBUG
-        cerr<<"show iconmenu("<<event.x_root<<", "<<event.y_root<<")"<<endl;
-#endif // DEBUG
         m_iconmenu.raise();
         m_iconmenu.move(event.x_root, event.y_root);
         m_iconmenu.show();
@@ -345,15 +280,17 @@ void FbDesk::buttonReleaseEvent(XButtonEvent &event) {
 }
 
 void FbDesk::exposeEvent(XExposeEvent &event) {
-    Icon *exposed_icon = findIcon(event.window);
+    Icon *exposed_icon = findIcon( event.window );
     if (exposed_icon != 0)
         exposed_icon->update();
 }
 
 void FbDesk::motionNotifyEvent(XMotionEvent &event) {
 
-    if (m_selected_icon == 0) // need a selected icon first
+    // need a selected icon first, and not locked position
+    if (*m_lock_positions || m_selected_icon == 0) 
         return;
+
     Display *disp = FbTk::App::instance()->display();
     Screen *scr = ScreenOfDisplay(disp, 0);
 
@@ -402,10 +339,6 @@ void FbDesk::clientMessageEvent(XClientMessageEvent &event) {
 }
 
 void FbDesk::execute(const std::string &commando) const {
-#ifdef DEBUG
-    cerr<<__FILE__<<" execute(\""<<commando<<"\");"<<endl;
-#endif // DEBUG
-
     if (! fork()) {
         string displaystring("DISPLAY=");
         displaystring += DisplayString(FbTk::App::instance()->display());
@@ -439,12 +372,12 @@ bool FbDesk::saveIcons(const std::string &filename) const {
     IconList::const_iterator it = m_iconlist.begin();
     IconList::const_iterator it_end = m_iconlist.end();
     for (; it != it_end; ++it) {		
-        ofile<<"[Desktop Entry]"<<endl;
-        ofile<<"Name="<<(*it)->label()<<endl;
-        ofile<<"Exec="<<(*it)->command()<<endl;
-        ofile<<"Icon="<<(*it)->imageFilename()<<endl;
-        ofile<<"Pos= "<<(*it)->x()<<" "<<(*it)->y()<<endl;
-        ofile<<"[end]"<<endl<<endl;
+        ofile<<"[Desktop Entry]"<<endl
+             <<"Name="<<(*it)->label()<<endl
+             <<"Exec="<<(*it)->command()<<endl
+             <<"Icon="<<(*it)->imageFilename()<<endl
+             <<"Pos= "<<(*it)->x()<<" "<<(*it)->y()<<endl
+             <<"[end]"<<endl<<endl;
     }
 
     return true;
@@ -464,21 +397,26 @@ void FbDesk::deleteIcons() {
 void FbDesk::reloadConfig() {
 
     m_resmanager.load(StringUtil::expandFilename(m_config_filename).c_str());
-    m_resmanager.loadItem(StringUtil::expandFilename(m_config_filename).c_str(),
-                          "session.styleFile", "Session.StyleFile", m_themefile);
-    if (m_themefile.empty()) {
-        m_resmanager.loadItem(StringUtil::expandFilename("~/.fluxbox/init").c_str(), 
-                              "session.styleFile", 
-                              "Session.StyleFile", m_themefile);
+    // if style file is empty read fluxbox style
+    if ( (*m_themefile).empty() ) {
+        string filename(StringUtil::expandFilename("~/.fluxbox/init").c_str());
+        ResourceManager rm(filename.c_str(), false);
+        rm.addResource( m_themefile );
+        rm.load(filename.c_str()); 
     }
-    cerr<<m_themefile<<endl;
 
     m_textcolor.setFromString(m_textcolorname->c_str(), 0);
+    if (!m_text_background_color.
+        setFromString(m_textbackgroundname->c_str(), 0)) {
+        m_text_background_color.setFromString("white", 0);
+    }
+
     // setup icon values
-    Icon::setTextColor(m_textcolor);
-    Icon::setTextPlacement(*m_textplacement);
-    Icon::setTextAlpha(*m_text_alpha);
-    Icon::setIconAlpha(*m_icon_alpha);
+    m_icon_config.setTextColor(m_textcolor);
+    m_icon_config.setTextBackground(m_text_background_color);
+    m_icon_config.setTextPlacement(*m_textplacement);
+    m_icon_config.setTextAlpha(*m_text_alpha);
+    m_icon_config.setIconAlpha(*m_icon_alpha);
 
     if (*m_textplacement == Icon::TEXTPLACE_LEFT ||
         *m_textplacement == Icon::TEXTPLACE_RIGHT)
@@ -502,9 +440,6 @@ void FbDesk::createIcon() {
 }
 
 void FbDesk::deleteLastIcon() {
-#ifdef DEBUG
-    cerr<<"Delete last icon"<<endl;
-#endif // DEBUG
     if (m_last_icon != 0 && 
         find(m_iconlist.begin(), m_iconlist.end(), m_last_icon) != m_iconlist.end()) {
         m_iconlist.remove(m_last_icon);
@@ -560,16 +495,42 @@ void FbDesk::setCommandFromInput() {
     m_inputfield.hide();
 }
 
+
+class ToggleMenuItem:public MenuItem {
+public:
+    explicit ToggleMenuItem(const char *label,
+                            bool &val ):
+        MenuItem(label),
+        m_value( val ) {
+        setToggleItem(true);
+    }  
+    void click(int button, int time) { 
+        setSelected(!m_value); 
+        FbTk::MenuItem::click(button, time); 
+    }
+
+    void setSelected(bool val) {
+        m_value = val;
+    }
+    bool isSelected() const {
+        return m_value;
+    }
+
+private:
+    bool &m_value;
+};
+
 void FbDesk::setupIconMenu() {
     using namespace FbTk;
     // setup menu items
     typedef RefCount<Command> CommandRef;
     //    RefCount<Command> show_iconbrowser(new SimpleCommand<IconBrowser>(m_iconbrowser, &IconBrowser::show));
-    CommandRef new_icon(new SimpleCommand<FbDesk>(*this, &FbDesk::createIcon));
-    CommandRef del_last_icon(new SimpleCommand<FbDesk>(*this, &FbDesk::deleteLastIcon));
-    CommandRef set_command(new SimpleCommand<FbDesk>(*this, &FbDesk::setIconCommand));
-    CommandRef set_label(new SimpleCommand<FbDesk>(*this, &FbDesk::setIconLabel));
-    CommandRef reload_style(new SimpleCommand<FbDesk>(*this, &FbDesk::reloadConfig));
+    typedef SimpleCommand<FbDesk> FbDeskCmd;
+    CommandRef new_icon(new FbDeskCmd(*this, &FbDesk::createIcon));
+    CommandRef del_last_icon(new FbDeskCmd(*this, &FbDesk::deleteLastIcon));
+    CommandRef set_command(new FbDeskCmd(*this, &FbDesk::setIconCommand));
+    CommandRef set_label(new FbDeskCmd(*this, &FbDesk::setIconLabel));
+    CommandRef reload_style(new FbDeskCmd(*this, &FbDesk::reloadConfig));
     CommandRef exit_fbdesk(new SimpleCommand<App>(*App::instance(), &App::end));
     //        select_icon_image_hide(select_icon_macro);
 
@@ -580,17 +541,22 @@ void FbDesk::setupIconMenu() {
     m_iconmenu.insert(new FbTk::MenuSeparator());
     m_iconmenu.insert("New icon", new_icon);
     m_iconmenu.insert("Delete icon", del_last_icon);
-    m_iconmenu.insert(new FbTk::MenuSeparator());
+    m_iconmenu.insert(new FbTk::MenuSeparator());    
     // setup alpha menu
-    FbTk::MenuItem *item = new IntResMenuItem("Text alpha", m_text_alpha, 0, 255);
+    FbTk::MenuItem *item = new IntResMenuItem("Text alpha", m_text_alpha, 
+                                              0, 255, 
+                                              m_iconmenu);
     CommandRef update_alpha(new FbTk::SimpleCommand<FbDesk>(*this, &FbDesk::updateAlpha));
     item->setCommand(update_alpha);
     m_iconmenu.insert(item);
-    item = new IntResMenuItem("Icon alpha", m_icon_alpha, 0, 255);
+    item = new IntResMenuItem("Icon alpha", m_icon_alpha, 
+                              0, 255,
+                              m_iconmenu);
     item->setCommand(update_alpha);
     m_iconmenu.insert(item);
-
     m_iconmenu.insert("Reload config", reload_style);
+    m_iconmenu.insert(new FbTk::MenuSeparator());    
+    m_iconmenu.insert(new ToggleMenuItem("Lock positions", *m_lock_positions) );
     m_iconmenu.insert(new FbTk::MenuSeparator());
     m_iconmenu.insert("Exit FbDesk", exit_fbdesk);
     //    m_iconmenu.insert("Select icon image", select_icon_image_hide);
@@ -600,8 +566,8 @@ void FbDesk::setupIconMenu() {
 void FbDesk::updateAlpha() {
     m_resmanager.save(StringUtil::expandFilename(m_config_filename).c_str(),
                       StringUtil::expandFilename(m_config_filename).c_str()); // merge
-    Icon::setTextAlpha(*m_text_alpha);
-    Icon::setIconAlpha(*m_icon_alpha);
+    m_icon_config.setTextAlpha(*m_text_alpha);
+    m_icon_config.setIconAlpha(*m_icon_alpha);
     updateIcons();
 }
 
@@ -612,3 +578,74 @@ void FbDesk::updateIcons() {
 }
 
 }; // end namespace FbDesk
+
+namespace FbTk {
+
+template <>
+void Resource<int>::setFromString(const char *str) {
+    sscanf(str, "%d", &*(*this));
+}
+
+template <>
+std::string Resource<int>::getString() const {
+    char strval[256];
+    sprintf(strval, "%d", *(*this));
+    return std::string(strval);
+}
+
+template <>
+void Resource<std::string>::setFromString(const char *str) {
+    *(*this) = (str ? str : "");
+}
+
+template <>
+std::string Resource<std::string>::getString() const {
+    return (*this)->c_str();
+}
+
+
+
+template <>
+std::string Resource<FbDesk::Icon::TextPlacement>::getString() const {
+    switch (*(*this)) {
+    case FbDesk::Icon::TEXTPLACE_LEFT:
+        return "Left";
+    case FbDesk::Icon::TEXTPLACE_RIGHT:
+        return "Right";
+    case FbDesk::Icon::TEXTPLACE_TOP:
+        return "Top";
+    case FbDesk::Icon::TEXTPLACE_BOTTOM:
+        return "Bottom";
+    }
+    return "Bottom";
+
+}
+
+template <>
+void Resource<FbDesk::Icon::TextPlacement>::setFromString(const char *str) {
+    if (strcmp("Left", str) == 0) {
+        *(*this) = FbDesk::Icon::TEXTPLACE_LEFT;
+    } else if (strcmp("Right", str) == 0) {
+        *(*this) = FbDesk::Icon::TEXTPLACE_RIGHT;
+    } else if (strcmp("Top", str) == 0) 
+        *(*this) = FbDesk::Icon::TEXTPLACE_TOP;
+    else  if (strcmp("Bottom", str) == 0)
+        *(*this) = FbDesk::Icon::TEXTPLACE_BOTTOM;
+    else 
+        *(*this) = FbDesk::Icon::TEXTPLACE_BOTTOM;
+}
+
+template<>
+std::string FbTk::Resource<bool>::
+getString() const {
+    return std::string(**this == true ? "true" : "false");
+}
+
+
+template<>
+void FbTk::Resource<bool>::
+setFromString(char const *strval) {
+    *this = (bool)!strcasecmp(strval, "true");
+}
+
+} //end namespace FbTk
