@@ -1,5 +1,5 @@
 // Color.cc for Fluxbox window manager
-// Copyright (c) 2002 - 2003 Henrik Kinnunen (fluxgen(at)users.sourceforge.net)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Color.cc,v 1.11 2004/06/07 11:46:05 rathnor Exp $
+// $Id: Color.cc 4199 2006-02-16 06:53:05Z mathias $
 
 #include "Color.hh"
 
@@ -30,11 +30,9 @@
 using namespace std;
 
 namespace {
-unsigned char maxValue(unsigned short colval) {
-    if (colval == 65535) 
-        return 0xFF;
 
-    return static_cast<unsigned char>(colval/0xFF);
+inline unsigned char maxValue(unsigned short colval) {
+   return colval == 65535 ? 0xFF : static_cast<unsigned char>(colval/0xFF);
 }
 
 };
@@ -105,6 +103,17 @@ bool Color::setFromString(const char *color_string, int screen) {
     return true;
 }
 
+bool Color::validColorString(const char *color_string, int screen) {
+    XColor color;
+    Display *disp = App::instance()->display();
+    Colormap colm = DefaultColormap(disp, screen);   
+    // trim white space
+    string color_string_tmp = color_string;
+    StringUtil::removeFirstWhitespace(color_string_tmp);
+    StringUtil::removeTrailingWhitespace(color_string_tmp);
+
+    return XParseColor(disp, colm, color_string_tmp.c_str(), &color) != 0;
+}
 
 Color &Color::operator  = (const Color &col_copy) {
     // check for aliasing
@@ -155,7 +164,7 @@ void Color::allocate(unsigned short red, unsigned short green, unsigned short bl
 	
 	
     if (!XAllocColor(disp, DefaultColormap(disp, screen), &color)) {
-        cerr<<"FbTk::Color: Color allocation error."<<endl;
+        cerr<<"FbTk::Color: Allocation error." << endl;
     } else {
         setRGB(maxValue(color.red),
                maxValue(color.green),
