@@ -1,5 +1,5 @@
 // EventManager.cc
-// Copyright (c) 2002 Henrik Kinnunen (fluxgen at linuxmail.org)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: EventManager.cc,v 1.14 2004/09/10 07:15:23 akir Exp $
+// $Id: EventManager.cc 4199 2006-02-16 06:53:05Z mathias $
 
 #include "EventManager.hh"
 #include "FbWindow.hh"
@@ -189,7 +189,7 @@ void EventManager::dispatch(Window win, XEvent &ev, bool parent) {
 
         if (parent_win != 0 &&
             parent_win != root) {
-            if (m_parent.find(parent_win) == m_parent.end())
+            if (m_parent[parent_win] == 0)
                 return;
 
             // dispatch event to parent
