@@ -1,5 +1,5 @@
 // FbWindow.cc for FbTk - fluxbox toolkit
-// Copyright (c) 2002-2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: FbWindow.cc,v 1.43 2004/09/12 14:56:19 rathnor Exp $
+// $Id: FbWindow.cc 4199 2006-02-16 06:53:05Z mathias $
 
 #include "FbWindow.hh"
 #include "FbPixmap.hh"
@@ -44,26 +44,28 @@
 
 namespace FbTk {
 
-FbWindow::FbWindow():FbDrawable(), m_parent(0), m_screen_num(0), m_window(0), m_x(0), m_y(0), 
+FbWindow::FbWindow():FbDrawable(), m_parent(0), m_screen_num(0), m_window(0), m_x(0), m_y(0),
                      m_width(0), m_height(0), m_border_width(0), m_depth(0), m_destroy(true),
-                     m_buffer_pm(0){
+                     m_lastbg_color_set(false), m_lastbg_color(0), m_lastbg_pm(0), m_renderer(0) {
 
 }
 
-FbWindow::FbWindow(const FbWindow& the_copy):m_parent(the_copy.parent()), 
-                                             m_screen_num(the_copy.screenNumber()), m_window(the_copy.window()), 
-                                             m_x(the_copy.x()), m_y(the_copy.y()), 
+FbWindow::FbWindow(const FbWindow& the_copy):FbDrawable(),
+                                             m_parent(the_copy.parent()),
+                                             m_screen_num(the_copy.screenNumber()), m_window(the_copy.window()),
+                                             m_x(the_copy.x()), m_y(the_copy.y()),
                                              m_width(the_copy.width()), m_height(the_copy.height()),
-                                             m_border_width(the_copy.borderWidth()), 
+                                             m_border_width(the_copy.borderWidth()),
                                              m_depth(the_copy.depth()), m_destroy(true),
-                                             m_buffer_pm(0) {
+                                             m_lastbg_color_set(false), m_lastbg_color(0), 
+                                             m_lastbg_pm(0), m_renderer(the_copy.m_renderer) {
     the_copy.m_window = 0;
 }
 
 FbWindow::FbWindow(int screen_num,
-                   int x, int y, 
-                   unsigned int width, unsigned int height, 
-                   long eventmask, 
+                   int x, int y,
+                   unsigned int width, unsigned int height,
+                   long eventmask,
                    bool override_redirect,
                    bool save_unders,
                    int depth,
@@ -72,31 +74,34 @@ FbWindow::FbWindow(int screen_num,
     m_parent(0),
     m_screen_num(screen_num),
     m_destroy(true),
-    m_buffer_pm(0) {
-	
-    create(RootWindow(display(), screen_num), 
+    m_lastbg_color_set(false),
+    m_lastbg_color(0),
+    m_lastbg_pm(0), m_renderer(0) {
+
+    create(RootWindow(display(), screen_num),
            x, y, width, height, eventmask,
            override_redirect, save_unders, depth, class_type);
 };
 
 FbWindow::FbWindow(const FbWindow &parent,
-                   int x, int y, unsigned int width, unsigned int height, 
+                   int x, int y, unsigned int width, unsigned int height,
                    long eventmask,
                    bool override_redirect,
                    bool save_unders,
                    int depth, int class_type):
     m_parent(&parent),
-    m_screen_num(parent.screenNumber()), 
+    m_screen_num(parent.screenNumber()),
     m_destroy(true),
-    m_buffer_pm(0) { 
+    m_lastbg_color_set(false), m_lastbg_color(0),
+    m_lastbg_pm(0), m_renderer(0) {
 
-    create(parent.window(), x, y, width, height, eventmask, 
+    create(parent.window(), x, y, width, height, eventmask,
            override_redirect, save_unders, depth, class_type);
-	
-	
+
+
 };
 
-FbWindow::FbWindow(Window client):FbDrawable(), m_parent(0), 
+FbWindow::FbWindow(Window client):FbDrawable(), m_parent(0),
                                   m_screen_num(0),
                                   m_window(0),
                                   m_x(0), m_y(0),
@@ -104,7 +109,8 @@ FbWindow::FbWindow(Window client):FbDrawable(), m_parent(0),
                                   m_border_width(0),
                                   m_depth(0),
                                   m_destroy(false),  // don't destroy this window
-                                  m_buffer_pm(0) {
+                                  m_lastbg_color_set(false), m_lastbg_color(0),
+                                  m_lastbg_pm(0), m_renderer(0) {
 
     setNew(client);
 }
@@ -117,26 +123,114 @@ FbWindow::~FbWindow() {
 
     if (m_window != 0) {
         // so we don't get any dangling eventhandler for this window
-        FbTk::EventManager::instance()->remove(m_window); 
+        FbTk::EventManager::instance()->remove(m_window);
         if (m_destroy)
-            XDestroyWindow(display(), m_window);     
+            XDestroyWindow(display(), m_window);
     }
 }
 
-
 void FbWindow::setBackgroundColor(const FbTk::Color &bg_color) {
-    XSetWindowBackground(display(), m_window, bg_color.pixel());
+    if (bg_color.isAllocated()) {
+        m_lastbg_color = bg_color.pixel();
+        m_lastbg_color_set = true;
+        m_lastbg_pm = None;
+    } else {
+        m_lastbg_color_set = false;
+    }
+
+    updateBackground(false);
 }
 
 void FbWindow::setBackgroundPixmap(Pixmap bg_pixmap) {
-    XSetWindowBackgroundPixmap(display(), m_window, bg_pixmap);
+    m_lastbg_pm = bg_pixmap;
+    if (bg_pixmap != None)
+        m_lastbg_color_set = false;
+
+    updateBackground(false);
+}
+
+void FbWindow::updateBackground(bool only_if_alpha) {
+    Pixmap newbg = m_lastbg_pm;
+    unsigned char alpha = 255;
+    bool free_newbg = false;
+
+    if (m_lastbg_pm == None && !m_lastbg_color_set)
+        return;
+
+    if (m_transparent.get() != 0)
+        alpha = m_transparent->alpha();
+
+    if (only_if_alpha && alpha == 255) 
+        return;
+
+    // still use bg buffer pixmap if not transparent
+    // cause it does nice caching things, assuming we have a renderer
+    if (m_lastbg_pm != ParentRelative && (m_renderer || alpha != 255)) {
+        // update source and destination if needed
+        Pixmap root = FbPixmap::getRootPixmap(screenNumber());
+        if (alpha != 255 && m_transparent->source() != root)
+            m_transparent->setSource(root, screenNumber());
+
+        FbPixmap newpm = FbPixmap(*this, width(), height(), depth());
+        free_newbg = true; // newpm gets released to newbg at end of block
+        GC gc = XCreateGC(display(), window(), 0, 0);
+
+        if (m_lastbg_pm == None && m_lastbg_color_set) {
+            XSetForeground(display(), gc, m_lastbg_color);
+            newpm.fillRectangle(gc, 0, 0, width(), height());
+        } else {
+            // copy from window if no color and no bg...
+            newpm.copyArea((m_lastbg_pm == None)?drawable():m_lastbg_pm, gc, 0, 0, 0, 0, width(), height());
+        }
+        XFreeGC(display(), gc);
+
+        if (alpha != 255)
+            m_transparent->setDest(newpm.drawable(), screenNumber());
+
+        // get root position
+
+        const FbWindow *root_parent = parent();
+        // our position in parent ("root")
+        int root_x = x() + borderWidth(), root_y = y() + borderWidth();
+        if (root_parent != 0) {
+            root_x += root_parent->x() + root_parent->borderWidth();
+            root_y += root_parent->y() + root_parent->borderWidth();
+            while (root_parent->parent() != 0) {
+                root_parent = root_parent->parent();
+                root_x += root_parent->x() + root_parent->borderWidth();
+                root_y += root_parent->y() + root_parent->borderWidth();
+            }
+        }
+
+        // render background image from root pos to our window
+        if (alpha != 255)
+            m_transparent->render(root_x, root_y,
+                                  0, 0,
+                                  width(), height());
+
+        // render any foreground items
+        if (m_renderer)
+            m_renderer->renderForeground(*this, newpm);
+
+        if (alpha != 255)
+            m_transparent->freeDest(); // it's only temporary, don't leave it hanging around
+        newbg = newpm.release();
+    }
+
+    if (newbg != None)
+        XSetWindowBackgroundPixmap(display(), m_window, newbg);
+    else if (m_lastbg_color_set)
+        XSetWindowBackground(display(), m_window, m_lastbg_color);
+
+    if (free_newbg)
+        XFreePixmap(display(), newbg);
 }
 
 void FbWindow::setBorderColor(const FbTk::Color &border_color) {
     XSetWindowBorder(display(), m_window, border_color.pixel());
 }
 
-void FbWindow::setBorderWidth(unsigned int size) {	
+void FbWindow::setBorderWidth(unsigned int size) {
     XSetWindowBorderWidth(display(), m_window, size);
     m_border_width = size;
 }
@@ -153,13 +247,18 @@ void FbWindow::clear() {
     XClearWindow(display(), m_window);
 }
 
-void FbWindow::clearArea(int x, int y, 
-                         unsigned int width, unsigned int height, 
+void FbWindow::clearArea(int x, int y,
+                         unsigned int width, unsigned int height,
                          bool exposures) {
+    // TODO: probably could call renderForeground here (with x,y,w,h)
     XClearArea(display(), window(), x, y, width, height, exposures);
 }
 
-void FbWindow::updateTransparent(int the_x, int the_y, unsigned int the_width, unsigned int the_height) {
+// If override_is_offset, then dest_override is a pixmap located at the_x, the_y
+// with size the_width x the_height  in the target window.
+
+void FbWindow::updateTransparent(int the_x, int the_y, unsigned int the_width, 
+                                 unsigned int the_height, Pixmap dest_override, bool override_is_offset) {
 #ifdef HAVE_XRENDER
     if (!m_transparent.get())
         return;
@@ -167,6 +266,18 @@ void FbWindow::updateTransparent(int the_x, int the_y, unsigned int the_width, u
     if (width() == 0 || height() == 0)
         return;
 
+    if (!dest_override && 
+        (the_width == 0 && the_height == 0 || 
+         the_width == width() && the_height == height()) &&
+        the_x <= 0 && the_y <= 0) {
+        // do the whole thing
+        updateBackground(true);
+        return;
+    }
+
+    if (!dest_override)
+        dest_override = window();
+
     if (the_width == 0 || the_height == 0) {
         the_width = width();
         the_height = height();
@@ -182,13 +293,8 @@ void FbWindow::updateTransparent(int the_x, int the_y, unsigned int the_width, u
     if (m_transparent->source() != root)
         m_transparent->setSource(root, screenNumber());
 
-    if (m_buffer_pm) {
-        if (m_transparent->dest() != m_buffer_pm) {
-            m_transparent->setDest(m_buffer_pm, screenNumber());
-        }
-    } else if (m_transparent->dest() != window())
-        m_transparent->setDest(window(), screenNumber());
-
+    if (m_transparent->dest() != dest_override)
+        m_transparent->setDest(dest_override, screenNumber());
 
     // get root position
 
@@ -208,7 +314,7 @@ void FbWindow::updateTransparent(int the_x, int the_y, unsigned int the_width, u
 
     // render background image from root pos to our window
     m_transparent->render(root_x + the_x, root_y + the_y,
-                          the_x, the_y,
+                          override_is_offset?0:the_x, override_is_offset?0:the_y,
                           the_width, the_height);
 #endif // HAVE_XRENDER
 }
@@ -221,12 +327,19 @@ void FbWindow::setAlpha(unsigned char alpha) {
 
         // don't setOpaque, let controlling objects do that
         // since it's only needed on toplevel windows
-    } else if (m_transparent.get() == 0 && alpha < 255) {
-        m_transparent.reset(new Transparent(FbPixmap::getRootPixmap(screenNumber()), window(), alpha, screenNumber()));
-    } else if (alpha < 255 && alpha != m_transparent->alpha())
-        m_transparent->setAlpha(alpha);
-    else if (alpha == 255)
-        m_transparent.reset(0); // destroy transparent object
+    } else {
+        if (!FbTk::Transparent::haveRender()) 
+            alpha = 255;
+
+        if (m_transparent.get() == 0 && alpha < 255) {
+            m_transparent.reset(new Transparent(FbPixmap::getRootPixmap(screenNumber()), 
+                                                window(), 
+                                                alpha, screenNumber()));
+        } else if (alpha < 255 && alpha != m_transparent->alpha())
+            m_transparent->setAlpha(alpha);
+        else if (alpha == 255)
+            m_transparent.reset(0); // destroy transparent object
+    }
 #endif // HAVE_XRENDER
 }
 
@@ -254,7 +367,7 @@ FbWindow &FbWindow::operator = (const FbWindow &win) {
 }
 
 FbWindow &FbWindow::operator = (Window win) {
-    setNew(win);	
+    setNew(win);
     return *this;
 }
 
@@ -289,7 +402,7 @@ void FbWindow::setNew(Window win) {
             m_depth = attr.depth;
             m_border_width = attr.border_width;
         }
-        
+
     }
 }
 
@@ -318,7 +431,7 @@ void FbWindow::setInputFocus(int revert_to, int time) {
 }
 
 void FbWindow::setCursor(Cursor cur) {
-    XDefineCursor(display(), window(), cur); 
+    XDefineCursor(display(), window(), cur);
 }
 
 void FbWindow::unsetCursor() {
@@ -368,8 +481,8 @@ bool FbWindow::property(Atom property,
                         unsigned long *nitems_return,
                         unsigned long *bytes_after_return,
                         unsigned char **prop_return) const {
-    if (XGetWindowProperty(display(), window(), 
-                           property, long_offset, long_length, do_delete, 
+    if (XGetWindowProperty(display(), window(),
+                           property, long_offset, long_length, do_delete,
                            req_type, actual_type_return,
                            actual_format_return, nitems_return,
                            bytes_after_return, prop_return) == Success)
@@ -383,19 +496,23 @@ void FbWindow::changeProperty(Atom property, Atom type,
                               int mode,
                               unsigned char *data,
                               int nelements) {
-    
+
     XChangeProperty(display(), m_window, property, type,
-                    format, mode, 
+                    format, mode,
                     data, nelements);
 }
 
+void FbWindow::deleteProperty(Atom property) {
+    XDeleteProperty(display(), m_window, property);
+}
+
 int FbWindow::screenNumber() const {
     return m_screen_num;
 }
 
 long FbWindow::eventMask() const {
     XWindowAttributes attrib;
-    XGetWindowAttributes(display(), window(), 
+    XGetWindowAttributes(display(), window(),
                          &attrib);
     return attrib.your_event_mask;
 
@@ -409,10 +526,6 @@ void FbWindow::setOpaque(unsigned char alpha) {
 #endif // HAVE_XRENDER
 }
 
-void FbWindow::setBufferPixmap(Pixmap pm) {
-    m_buffer_pm = pm;
-}
-
 void FbWindow::updateGeometry() {
     if (m_window == 0)
         return;
@@ -420,16 +533,16 @@ void FbWindow::updateGeometry() {
     Window root;
     unsigned int border_width, depth;
     XGetGeometry(display(), m_window, &root, &m_x, &m_y,
-                 (unsigned int *)&m_width, (unsigned int *)&m_height, 
+                 (unsigned int *)&m_width, (unsigned int *)&m_height,
                  &border_width, &depth);
     m_depth = depth;
 }
 
 void FbWindow::create(Window parent, int x, int y,
-                      unsigned int width, unsigned int height, 
+                      unsigned int width, unsigned int height,
                       long eventmask, bool override_redirect,
                       bool save_unders, int depth, int class_type) {
-                     
+
 
     m_border_width = 0;
 
@@ -437,7 +550,7 @@ void FbWindow::create(Window parent, int x, int y,
     XSetWindowAttributes values;
     values.event_mask = eventmask;
 
-    if (override_redirect) {        
+    if (override_redirect) {
         valmask |= CWOverrideRedirect;
         values.override_redirect = True;
     }
@@ -448,17 +561,40 @@ void FbWindow::create(Window parent, int x, int y,
     }
 
     m_window = XCreateWindow(display(), parent, x, y, width, height,
-                             0, // border width  
+                             0, // border width
                              depth, // depth
                              class_type, // class
                              CopyFromParent, // visual
                              valmask, // create mask
                              &values); // create atrribs
-    
+
     assert(m_window);
 
     updateGeometry();
-    FbWindow::setBackgroundColor(Color("gray", screenNumber()));
+}
+
+
+void FbWindow::sendConfigureNotify(int x, int y, 
+                                   unsigned int width, unsigned int height) {
+    Display *disp = FbTk::App::instance()->display();
+    XEvent event;
+    event.type = ConfigureNotify;
+
+    event.xconfigure.display = disp;
+    event.xconfigure.event = window();
+    event.xconfigure.window = window();
+    event.xconfigure.x = x;
+    event.xconfigure.y = y;
+    event.xconfigure.width = width;
+    event.xconfigure.height = height;
+    //!! TODO
+    event.xconfigure.border_width = 1;
+    //!! TODO
+    event.xconfigure.above = None; 
+    event.xconfigure.override_redirect = false;
+
+    XSendEvent(disp, window(), False, StructureNotifyMask, &event);
+
 }
 
 bool operator == (Window win, const FbWindow &fbwin) {
