@@ -1,5 +1,5 @@
 // ImageControl.cc for FbTk - Fluxbox Toolkit
-// Copyright (c) 2001 - 2003 Henrik Kinnunen (fluxbox at users.sourceforge.net)
+// Copyright (c) 2001 - 2006 Henrik Kinnunen (fluxbox at users.sourceforge.net)
 //
 // Image.cc for Blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes at tcac.net)
@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ImageControl.cc,v 1.16 2004/09/10 16:41:30 akir Exp $
+// $Id: ImageControl.cc 4199 2006-02-16 06:53:05Z mathias $
 
 #include "ImageControl.hh"
 
@@ -30,7 +30,6 @@
 #include "App.hh"
 #include "SimpleCommand.hh"
 
-
 //use GNU extensions
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE
@@ -653,7 +652,7 @@ void ImageControl::createColorTable() {
         }
 
     default:
-        cerr<<"FbTk::ImageControl: Unsupported visual"<<endl;
+        cerr<<"FbTk::ImageControl: Unsupported visual."<<endl;
         break;
     }
 }
