@@ -1,5 +1,5 @@
 // StringUtil.cc for fluxbox
-// Copyright (c) 2001 - 2004 Henrik Kinnunen (fluxgen<at>fluxbox<dot>org)
+// Copyright (c) 2001 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,11 +19,11 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-//$Id: StringUtil.cc,v 1.13 2004/09/10 16:04:16 akir Exp $
+//$Id: StringUtil.cc 4199 2006-02-16 06:53:05Z mathias $
 
 #include "StringUtil.hh"
 
-#include <string>
+
 #ifdef HAVE_CSTDIO
   #include <cstdio>
 #else
@@ -44,10 +44,14 @@
 #else
   #include <assert.h>
 #endif
+
+
 #include <memory>
 #include <algorithm>
+#include <string>
 
-using namespace std;
+using std::string;
+using std::transform;
 
 namespace FbTk {
 
@@ -89,11 +93,11 @@ const char *strcasestr(const char *str, const char *ptn) {
    if ~ then expand it to home of user
    returns expanded filename
 */
-string expandFilename(const std::string &filename) {
+string expandFilename(const string &filename) {
     string retval;
     size_t pos = filename.find_first_not_of(" \t");
-    if (pos != std::string::npos && filename[pos] == '~') {
-    	retval = getenv("HOME");
+    if (pos != string::npos && filename[pos] == '~') {
+        retval = getenv("HOME");
         if (pos != filename.size()) {
             // copy from the character after '~'
             retval += static_cast<const char *>(filename.c_str() + pos + 1);
@@ -107,15 +111,36 @@ string expandFilename(const std::string &filename) {
 /**
    @return string from last "." to end of string
 */
-string findExtension(const std::string &filename) {
+string findExtension(const string &filename) {
     //get start of extension
-    std::string::size_type start_pos = filename.find_last_of(".");
-    if (start_pos == std::string::npos && start_pos != filename.size())
+    string::size_type start_pos = filename.find_last_of(".");
+    if (start_pos == string::npos && start_pos != filename.size())
         return "";
     // return from last . to end of string
     return filename.substr(start_pos + 1);
 }
 
+string replaceString(const string &original,
+                     const char *findthis,
+                     const char *replace) {
+    int i=0;
+    const int size_of_replace = strlen(replace);
+    const int size_of_find = strlen(findthis);
+    string ret_str(original);
+    while (i < ret_str.size()) {
+        i = ret_str.find(findthis, i);
+        if (i == string::npos)
+            break;
+        // erase old string and insert replacement
+        ret_str.erase(i, size_of_find);
+        ret_str.insert(i, replace);
+        // jump to next position after insert
+        i += size_of_replace;
+    }
+
+    return ret_str;
+}
+
 /**
    Parses a string between "first" and "last" characters
    and ignoring ok_chars as whitespaces. The value is
@@ -126,31 +151,31 @@ string findExtension(const std::string &filename) {
    for the position + 1 in the in-string where the "last"-char value
    was found.
 */
-int getStringBetween(std::string& out, const char *instr, const char first, const char last,
+int getStringBetween(string& out, const char *instr, const char first, const char last,
                      const char *ok_chars, bool allow_nesting) {
     assert(first);
     assert(last);
     assert(instr);
 
-    std::string::size_type i = 0,
-        total_add=0; //used to add extra if there is a \last to skip
-    std::string in(instr);
+    string::size_type i = 0;
+    string::size_type total_add=0; //used to add extra if there is a \last to skip
+    string in(instr);
 
     // eat leading whitespace
     i = in.find_first_not_of(ok_chars);
-    if (i == std::string::npos)
+    if (i == string::npos)
         return -in.size();   // nothing left but whitespace
 
     if (in[i]!=first)
         return -i; //return position to error
 
     // find the end of the token
-    std::string::size_type j = i, k;
+    string::size_type j = i, k;
     int nesting = 0;
     while (1) {
         k = in.find_first_of(first, j+1);
         j = in.find_first_of(last, j+1);
-        if (j==std::string::npos)
+        if (j==string::npos)
             return -in.size(); //send negative size
 
         if (allow_nesting && k < j && in[k-1] != '\\') {
@@ -175,26 +200,26 @@ int getStringBetween(std::string& out, const char *instr, const char first, cons
     return (j+1+total_add);
 }
 
-std::string toLower(const std::string &conv) {
-    std::string ret = conv;
-    std::transform(ret.begin(), ret.end(), ret.begin(), tolower);
+string toLower(const string &conv) {
+    string ret = conv;
+    transform(ret.begin(), ret.end(), ret.begin(), tolower);
     return ret;
 }
 
-std::string toUpper(const std::string &conv) {
-    std::string ret = conv;
-    std::transform(ret.begin(), ret.end(), ret.begin(), toupper);
+string toUpper(const string &conv) {
+    string ret = conv;
+    transform(ret.begin(), ret.end(), ret.begin(), toupper);
     return ret;
 }
 
-std::string basename(const std::string &filename) {
-    std::string::size_type first_pos = filename.find_last_of("/");
-    if (first_pos != std::string::npos)
+string basename(const string &filename) {
+    string::size_type first_pos = filename.find_last_of("/");
+    if (first_pos != string::npos)
         return filename.substr(first_pos + 1);
     return filename;
 }
 
-string::size_type removeFirstWhitespace(std::string &str) {
+string::size_type removeFirstWhitespace(string &str) {
     string::size_type first_pos = str.find_first_not_of(" \t");
     if (first_pos != string::npos)
         str.erase(0, first_pos);
@@ -202,7 +227,7 @@ string::size_type removeFirstWhitespace(std::string &str) {
 }
 
 
-string::size_type removeTrailingWhitespace(std::string &str) {
+string::size_type removeTrailingWhitespace(string &str) {
     // strip trailing whitespace
     string::size_type first_pos = str.find_last_not_of(" \t");
     if (first_pos != string::npos) {
