@@ -1,5 +1,5 @@
 // Texture.hh for Fluxbox Window Manager 
-// Copyright (c) 2002 Henrik Kinnunen (fluxgen@users.sourceforge.net)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // from Image.cc for Blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes@tcac.net)
@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Texture.cc,v 1.10 2004/08/31 15:26:39 rathnor Exp $
+// $Id: Texture.cc 4199 2006-02-16 06:53:05Z mathias $
 
 #include "App.hh"
 #include "Texture.hh"
@@ -57,9 +57,7 @@ void Texture::setFromString(const char * const texture_str) {
     } else {
         setType(Texture::NONE);
 
-        if (strstr(ts, "solid"))
-            addType(Texture::SOLID);
-        else if (strstr(ts, "gradient")) {
+        if (strstr(ts, "gradient")) {
             addType(Texture::GRADIENT);
             if (strstr(ts, "crossdiagonal"))
                 addType(Texture::CROSSDIAGONAL);
@@ -79,17 +77,15 @@ void Texture::setFromString(const char * const texture_str) {
                 addType(Texture::VERTICAL);
             else
                 addType(Texture::DIAGONAL);
-        } else
+        } else // default is "solid", according to ThemeItems.cc
             addType(Texture::SOLID);
 
         if (strstr(ts, "raised"))
             addType(Texture::RAISED);
         else if (strstr(ts, "sunken"))
             addType(Texture::SUNKEN);
-        else if (strstr(ts, "flat"))
-            addType(Texture::FLAT);        
-        else
-            addType(Texture::RAISED);
+        else // default us "flat", according to ThemeItems.cc
+            addType(Texture::FLAT);
 
         if (! (type() & Texture::FLAT))
             if (strstr(ts, "bevel2"))
