@@ -1,5 +1,5 @@
 // TextureRender.cc for fluxbox
-// Copyright (c) 2002 - 2004 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // from Image.cc for Blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes at tcac.net)
@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: TextureRender.cc,v 1.11 2004/08/31 15:26:39 rathnor Exp $
+// $Id: TextureRender.cc 4263 2006-03-20 22:23:41Z fluxgen $
 
 #include "TextureRender.hh"
 
@@ -40,6 +40,13 @@
 #endif
 using namespace std;
 
+// mipspro has no new(nothrow)
+#if defined sgi && ! defined GCC
+#define FB_new_nothrow new
+#else
+#define FB_new_nothrow new(std::nothrow)
+#endif
+
 namespace FbTk {
 
 TextureRender::TextureRender(ImageControl &imgctrl, 
@@ -48,46 +55,27 @@ TextureRender::TextureRender(ImageControl &imgctrl,
     control(imgctrl),
     colors(_colors),
     ncolors(ncolors),
+    cpc(imgctrl.colorsPerChannel()),
+    cpccpc(cpc * cpc),
+    red(0), green(0), blue(0),
+    width(static_cast<signed>((w > 0 ? w : 1))), height(static_cast<signed>(h > 0 ? h : 1)),
     xtable(0), ytable(0) {
 
-
-    width = ((signed) w > 0) ? w : 1;
-    height = ((signed) h > 0) ? h : 1;
+    unsigned int texture_max_width = WidthOfScreen(ScreenOfDisplay(FbTk::App::instance()->display(), imgctrl.screenNumber())) * 2;
+    unsigned int texture_max_height = HeightOfScreen(ScreenOfDisplay(FbTk::App::instance()->display(), imgctrl.screenNumber())) * 2;
+    
+ 
     // clamp to "normal" size
-    if (width > 3200) {
-        cerr<<"TextureRender: "<<"Warning! Width > 3200 setting Width = 3200"<<endl;
-        width = 3200;
+    if (width > texture_max_width) {
+        cerr<<"TextureRender: Warning! Width > 3200 setting Width = 3200"<<endl;
+        width = texture_max_width;
     }
 
-    if (height > 3200) {
-        cerr<<"TextureRender: "<<"Warning! Height > 3200 setting Height = 3200"<<endl;
-        height = 3200;
+    if (height > texture_max_height) {
+        cerr<<"TextureRender: Warning! Height > 3200 setting Height = 3200"<<endl;
+        height = texture_max_height;
     }
 
-    red = new(nothrow) unsigned char[width * height];
-    if (red == 0) {
-        char sbuf[128];
-        sprintf(sbuf, "%d", width*height);
-        throw string("TextureRender::TextureRender(): " + string("Out of memory while allocating red buffer.") + string(sbuf));
-    }
-
-
-    green = new(nothrow) unsigned char[width * height];
-    if (green == 0) {
-        char sbuf[128];
-        sprintf(sbuf, "%d", width*height);
-        throw string("TextureRender::TextureRender(): " + string("Out of memory while allocating red buffer.") + string(sbuf));
-    }
-
-    blue = new(nothrow) unsigned char[width * height];
-    if (blue == 0) {
-        char sbuf[128];
-        sprintf(sbuf, "%d", width*height);
-        throw string("TextureRender::TextureRender(): " + string("Out of memory while allocating red buffer.") + string(sbuf));
-    }
-
-    cpc = imgctrl.colorsPerChannel();
-    cpccpc = cpc * cpc;
 
     imgctrl.colorTables(&red_table, &green_table, &blue_table,
                         &red_offset, &green_offset, &blue_offset,
@@ -97,9 +85,9 @@ TextureRender::TextureRender(ImageControl &imgctrl,
 
 
 TextureRender::~TextureRender() {
-    delete [] red;
-    delete [] green;
-    delete [] blue;	
+    if (red != 0) delete [] red;
+    if (green != 0) delete [] green;
+    if (blue != 0) delete [] blue;	
 }
 
 
@@ -110,12 +98,45 @@ Pixmap TextureRender::render(const FbTk::Texture &texture) {
         return ParentRelative;
     else if (texture.type() & FbTk::Texture::SOLID)
         return renderSolid(texture);
-    else if (texture.type() & FbTk::Texture::GRADIENT)
+    else if (texture.type() & FbTk::Texture::GRADIENT) {
+        allocateColorTables();
         return renderGradient(texture);
+    }
 
     return None;
 }
 
+void TextureRender::allocateColorTables() {
+
+    const size_t size = width * height;
+    red = FB_new_nothrow unsigned char[size];
+
+    if (red == 0) {
+        char sbuf[128];
+        sprintf(sbuf, "%ld", (long int) size);
+        throw std::string("TextureRender::TextureRender(): " + 
+                          std::string("Out of memory while allocating red buffer.") + string(sbuf));
+    }
+
+
+    green = FB_new_nothrow unsigned char[size];
+    if (green == 0) {
+        char sbuf[128];
+        sprintf(sbuf, "%ld", (long int) size);
+        throw std::string("TextureRender::TextureRender(): " + 
+              std::string("Out of memory while allocating green buffer.") + string(sbuf));
+    }
+
+    blue = FB_new_nothrow unsigned char[size];
+    if (blue == 0) {
+        char sbuf[128];
+        sprintf(sbuf, "%ld", (long int) size);
+        throw std::string("TextureRender::TextureRender(): " +
+              std::string("Out of memory while allocating blue buffer.")+ string(sbuf));
+    }
+
+
+}
 
 Pixmap TextureRender::renderSolid(const FbTk::Texture &texture) {
 
@@ -283,7 +304,7 @@ XImage *TextureRender::renderXImage() {
                      width, height, 32, 0);
 
     if (! image) {
-        cerr<<"FbTk::TextureRender::renderXImage(): Error creating XImage"<<endl;
+        cerr << "FbTk::TextureRender::renderXImage(): Can't create XImage." << endl;
         return 0;
     }
 
@@ -297,7 +318,7 @@ XImage *TextureRender::renderXImage() {
 
     o = image->bits_per_pixel + ((image->byte_order == MSBFirst) ? 1 : 0);
 
-    if (control.doDither() && width > 1 && height > 1) {
+    if (control.doDither()) {
         unsigned char dither4[4][4] = { 
             {0, 4, 1, 5},
             {6, 2, 7, 3},
@@ -554,12 +575,14 @@ XImage *TextureRender::renderXImage() {
         */
 
     default:
-        cerr<<"TextureRender::renderXImage(): unsupported visual"<<endl;
+        cerr << "TextureRender::renderXImage(): Unsupported visual" << endl;
         delete [] d;
         XDestroyImage(image);
         return (XImage *) 0;
     }
-} else {
+} else { // end do dither
+
+    // no dither:
     switch (control.visual()->c_class) {
     case StaticColor:
     case PseudoColor:
@@ -653,7 +676,7 @@ XImage *TextureRender::renderXImage() {
 	break;
 
     default:
-        cerr<<"TextureRender::renderXImage(): unsupported visual"<<endl;
+        cerr << "TextureRender::renderXImage(): Unsupported visual"<< endl;
         delete [] d;
         XDestroyImage(image);
         return (XImage *) 0;
