@@ -1,5 +1,5 @@
 // Theme.cc for FbTk - Fluxbox ToolKit
-// Copyright (c) 2002 - 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,15 +19,15 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Theme.cc,v 1.29 2004/08/31 15:26:39 rathnor Exp $
+// $Id: Theme.cc 4199 2006-02-16 06:53:05Z mathias $
 
 #include "Theme.hh"
 
 #include "XrmDatabaseHelper.hh"
 #include "App.hh"
 #include "StringUtil.hh"
-#include "ThemeItems.hh"
-#include "Directory.hh"
+#include "FileUtil.hh"
+#include "Image.hh"
 
 #ifdef HAVE_CSTDIO
   #include <cstdio>
@@ -41,6 +41,27 @@ using namespace std;
 
 namespace FbTk {
 
+struct LoadThemeHelper {
+    LoadThemeHelper():m_tm(ThemeManager::instance()) {}
+    void operator ()(Theme *tm) {
+        m_tm.loadTheme(*tm);
+    }
+    void operator ()(ThemeManager::ThemeList &tmlist) {
+        
+        for_each(tmlist.begin(), tmlist.end(), 
+                 *this);
+        // send reconfiguration signal to theme and listeners
+        ThemeManager::ThemeList::iterator it = tmlist.begin();
+        ThemeManager::ThemeList::iterator it_end = tmlist.end();
+        for (; it != it_end; ++it) {
+            (*it)->reconfigTheme();
+            (*it)->reconfigSig().notify();
+        }
+    }
+
+    ThemeManager &m_tm;    
+};
+
 Theme::Theme(int screen_num):m_screen_num(screen_num) {
     ThemeManager::instance().registerTheme(*this);
 }
@@ -64,36 +85,44 @@ ThemeManager::ThemeManager():
 }
 
 bool ThemeManager::registerTheme(Theme &tm) {
-    if (m_max_screens < 0)
+    if (m_max_screens < 0) {
         m_max_screens = ScreenCount(FbTk::App::instance()->display());
+        m_themes.resize(m_max_screens);
+    }
 
     // valid screen num?
     if (m_max_screens < tm.screenNum() || tm.screenNum() < 0)
         return false;
     // TODO: use find and return false if it's already there
     // instead of unique 
-    m_themelist.push_back(&tm);
-    m_themelist.unique(); 
+
+    m_themes[tm.screenNum()].push_back(&tm);
+    m_themes[tm.screenNum()].unique();
     return true;
 }
 
 bool ThemeManager::unregisterTheme(Theme &tm) {
-    m_themelist.remove(&tm);
+    if (m_max_screens < tm.screenNum() || tm.screenNum() < 0)
+        return false;
+
+    m_themes[tm.screenNum()].remove(&tm);
+
     return true;
 }
 
-bool ThemeManager::load(const std::string &filename, int screen_num) {
+bool ThemeManager::load(const std::string &filename, 
+                        const std::string &overlay_filename, int screen_num) {
     std::string location = FbTk::StringUtil::expandFilename(filename);
     std::string prefix = "";
 
-    if (Directory::isDirectory(filename)) {
+    if (FileUtil::isDirectory(filename.c_str())) {
         prefix = location;
 
         location.append("/theme.cfg");
-        if (!Directory::isRegularFile(location)) {
+        if (!FileUtil::isRegularFile(location.c_str())) {
             location = prefix;
             location.append("/style.cfg");
-            if (!Directory::isRegularFile(location)) {
+            if (!FileUtil::isRegularFile(location.c_str())) {
                 cerr<<"Error loading theme file "<<location<<": not a regular file"<<endl;
                 return false;
             }
@@ -106,6 +135,20 @@ bool ThemeManager::load(const std::string &filename, int screen_num) {
     if (!m_database.load(location.c_str()))
         return false;
 
+
+    if (!overlay_filename.empty()) {
+        std::string overlay_location = FbTk::StringUtil::expandFilename(overlay_filename);
+        if (FileUtil::isRegularFile(overlay_location.c_str())) {
+            XrmDatabaseHelper overlay_db;
+            if (overlay_db.load(overlay_location.c_str())) {
+                // after a merge the src_db is destroyed
+                // so, make sure XrmDatabaseHelper::m_database == 0
+                XrmMergeDatabases(*overlay_db, &(*m_database));
+                *overlay_db = 0;
+            }
+        }
+    }
+
     // relies on the fact that load_rc clears search paths each time
     if (m_themelocation != "") {
         Image::removeSearchPath(m_themelocation);
@@ -120,29 +163,24 @@ bool ThemeManager::load(const std::string &filename, int screen_num) {
     location.append("/pixmaps");
     Image::addSearchPath(location);
 
+    LoadThemeHelper load_theme_helper;
+
     // get list and go throu all the resources and load them
-    ThemeList::iterator theme_it = m_themelist.begin();
-    const ThemeList::iterator theme_it_end = m_themelist.end();
-    for (; theme_it != theme_it_end; ++theme_it) {
-        if (screen_num < 0)
-            loadTheme(**theme_it);
-        else if (screen_num == (*theme_it)->screenNum()) // specified screen
-            loadTheme(**theme_it);
-            
-    }
-    // notify all themes that we reconfigured
-    theme_it = m_themelist.begin();
-    for (; theme_it != theme_it_end; ++theme_it) {
-        // send reconfiguration signal to theme and listeners
-        (*theme_it)->reconfigTheme();
-        (*theme_it)->reconfigSig().notify();
+    // and then reconfigure them
+    if (screen_num < 0 || screen_num > m_max_screens) {
+        for_each(m_themes.begin(),
+                 m_themes.end(),
+                 load_theme_helper);
+    } else {
+        load_theme_helper(m_themes[screen_num]);
     }
+
     return true;
 }
 
 void ThemeManager::loadTheme(Theme &tm) {
-    std::list<ThemeItem_base *>::iterator i = tm.itemList().begin();
-    std::list<ThemeItem_base *>::iterator i_end = tm.itemList().end();
+    Theme::ItemList::iterator i = tm.itemList().begin();
+    Theme::ItemList::iterator i_end = tm.itemList().end();
     for (; i != i_end; ++i) {
         ThemeItem_base *resource = *i;
         if (!loadItem(*resource)) {
