@@ -1,5 +1,5 @@
 // Timer.cc for FbTk - Fluxbox Toolkit
-// Copyright (c) 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Timer.cc for Blackbox - An X11 Window Manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes at tcac.net)
@@ -110,6 +110,7 @@ void Timer::updateTimers(int fd) {
     FD_ZERO(&rfds);
     FD_SET(fd, &rfds);
 
+  
     if (m_timerlist.size() > 0) {
         gettimeofday(&now, 0);
 
@@ -142,13 +143,27 @@ void Timer::updateTimers(int fd) {
 
     select(fd + 1, &rfds, 0, 0, timeout);
 
+    TimerList::iterator it;
+
+    // someone set the date of the machine BACK
+    // so we have to adjust the start_time
+    static time_t last_time = time(0);
+    if (time(0) < last_time) {
+    
+        time_t delta = time(0) - last_time;
+
+        for (it = m_timerlist.begin(); it != m_timerlist.end(); it++) {
+            (*it)->m_start.tv_sec += delta;
+        }
+    }
+
+
     // check for timer timeout
     gettimeofday(&now, 0);
 
-    TimerList::iterator it = m_timerlist.begin();
     //must check end ...the timer might remove
     //it self from the list (should be fixed in the future)
-    for(; it != m_timerlist.end(); ++it) {
+    for(it = m_timerlist.begin(); it != m_timerlist.end(); ) {
         //This is to make sure we don't get an invalid iterator
         //when we do fireTimeout
         Timer &t = *(*it);
@@ -157,19 +172,30 @@ void Timer::updateTimers(int fd) {
         tm.tv_usec = t.getStartTime().tv_usec +
             t.getTimeout().tv_usec;
 
-        if ((now.tv_sec < tm.tv_sec) ||
-            (now.tv_sec == tm.tv_sec && now.tv_usec < tm.tv_usec))
+        if (((now.tv_sec < tm.tv_sec) ||
+             (now.tv_sec == tm.tv_sec && now.tv_usec < tm.tv_usec)))
             break;
 
         t.fireTimeout();
         // restart the current timer so that the start time is updated
-        if (! t.doOnce())
+        if (! t.doOnce()) {
             t.start();
-        else {
-            t.stop();			
-            it--;
-        }					
+
+            // Note that this mustn't be done if we're deleting the
+            // entry from the list, so therefore it's not in the update
+            // section of the for loop
+            it++;
+        } else {
+            // Since the default stop behaviour results in the timer
+            // being removed, we must remove it here, so that the iterator
+            // lives well. Another option would be to add it to another
+            // list, and then just go through that list and stop them all.
+            it = m_timerlist.erase(it);
+            t.stop();
+        }
     }
+
+    last_time = time(0);
 }
 
 void Timer::addTimer(Timer *timer) {
