@@ -1,5 +1,5 @@
 // XFontImp.cc for FbTk fluxbox toolkit
-// Copyright (c) 2002-2004 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 // 
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: XFontImp.cc,v 1.10 2004/09/11 22:58:20 fluxgen Exp $
+// $Id: XFontImp.cc 4262 2006-03-20 22:23:07Z fluxgen $
 
 #include "XFontImp.hh"
 #include "App.hh"
@@ -123,7 +123,6 @@ void XFontImp::rotate(float angle) {
         return;
     }
 
-
     //get positive angle
     while (angle < 0)
         angle += 360;
@@ -160,7 +159,7 @@ void XFontImp::rotate(float angle) {
     m_rotfont = new(nothrow) XRotFontStruct;
 
     if (m_rotfont == 0) {
-        cerr<<"RotFont: Out of memory"<<endl;
+        cerr<<"RotFont: Out of memory" << endl;
         return;
     }
    
@@ -219,7 +218,7 @@ void XFontImp::rotate(float angle) {
                          boxlen/2 - descent, text, 1);
 
         // reserve memory for first XImage
-        vertdata = new unsigned char[vert_len * vert_h];
+        vertdata = (unsigned char *)calloc((unsigned)(vert_len * vert_h), 1);
 
         XImage *I1 = XCreateImage(dpy, DefaultVisual(dpy, screen), 
                                   1, XYBitmap,
@@ -227,7 +226,8 @@ void XFontImp::rotate(float angle) {
                                   vert_w, vert_h, 8, 0);
 
         if (I1 == None) {				
-            cerr<<"RotFont: Cant create XImage"<<endl;
+            cerr << "RotFont: Can't create XImage."<<endl;
+            free(vertdata);
             delete m_rotfont;
             m_rotfont = 0;			
             return;
@@ -265,7 +265,8 @@ void XFontImp::rotate(float angle) {
                           (char *)bitdata, bit_w, bit_h, 8, 0); 
 
         if (I2 == None) {
-            cerr<<"XFontImp: Cant create XImage"<<endl;
+            cerr << "XFontImp: Can't create XImage."<<endl;
+            free(bitdata);
             delete m_rotfont;
             m_rotfont = 0;
             return;
