@@ -1,5 +1,5 @@
 // XmbFontImp.cc for FbTk fluxbox toolkit
-// Copyright (c) 2002-2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: XmbFontImp.cc,v 1.13 2004/09/11 22:58:20 fluxgen Exp $
+// $Id: XmbFontImp.cc 4199 2006-02-16 06:53:05Z mathias $
 
 #include "XmbFontImp.hh"
 
@@ -110,14 +110,12 @@ const char *getFontElement(const char *pattern, char *buf, int bufsiz, ...) {
     return 0;
 }
 
-XFontSet createFontSet(const char *fontname, bool utf8mode) {
+XFontSet createFontSet(const char *fontname, bool& utf8mode) {
     Display *display = FbTk::App::instance()->display();
     XFontSet fs;
-    const int FONT_ELEMENT_SIZE=50;
     char **missing, *def = "-";
-    int nmissing, pixel_size = 0, buf_size = 0;
-    char weight[FONT_ELEMENT_SIZE], slant[FONT_ELEMENT_SIZE];
-    char * orig_locale = "";
+    int nmissing;
+    std::string orig_locale = "";
 
 #ifdef HAVE_SETLOCALE
     if (utf8mode) {
@@ -131,74 +129,34 @@ XFontSet createFontSet(const char *fontname, bool utf8mode) {
     if (fs && (! nmissing)) {
 #ifdef HAVE_SETLOCALE
         if (utf8mode)
-            setlocale(LC_CTYPE, orig_locale);
+            setlocale(LC_CTYPE, orig_locale.c_str());
 #endif // HAVE_SETLOCALE
         return fs;
     }
 
 #ifdef HAVE_SETLOCALE
     if (! fs) {
-        if (nmissing) XFreeStringList(missing);
+        if (nmissing)
+            XFreeStringList(missing);
 
         setlocale(LC_CTYPE, "C");
         fs = XCreateFontSet(display, fontname,
                             &missing, &nmissing, &def);
-        setlocale(LC_CTYPE, orig_locale);
+        setlocale(LC_CTYPE, orig_locale.c_str());
     }
-#endif // HAVE_SETLOCALE
-
-    if (fs) {
-        XFontStruct **fontstructs;
-        char **fontnames;
-        XFontsOfFontSet(fs, &fontstructs, &fontnames);
-        fontname = fontnames[0];
-    }
-
-    getFontElement(fontname, weight, FONT_ELEMENT_SIZE,
-                   "-medium-", "-bold-", "-demibold-", "-regular-", NULL);
-    getFontElement(fontname, slant, FONT_ELEMENT_SIZE,
-                   "-r-", "-i-", "-o-", "-ri-", "-ro-", NULL);
-    getFontSize(fontname, &pixel_size);
-
-    if (! strcmp(weight, "*"))
-        strncpy(weight, "medium", FONT_ELEMENT_SIZE);
-    if (! strcmp(slant, "*"))
-        strncpy(slant, "r", FONT_ELEMENT_SIZE);
-    if (pixel_size < 3)
-        pixel_size = 3;
-    else if (pixel_size > 97)
-        pixel_size = 97;
-
-    buf_size = strlen(fontname) + (FONT_ELEMENT_SIZE * 2) + 64;
-    char *pattern2 = new char[buf_size];
-    snprintf(pattern2, buf_size - 1,
-             "%s,"
-             "-*-*-%s-%s-*-*-%d-*-*-*-*-*-*-*,"
-             "-*-*-*-*-*-*-%d-*-*-*-*-*-*-*,*",
-             fontname, weight, slant, pixel_size, pixel_size);
-    fontname = pattern2;
-
-    if (nmissing)
-        XFreeStringList(missing);
-    if (fs)
-        XFreeFontSet(display, fs);
-
-    fs = XCreateFontSet(display, fontname,
-                        &missing, &nmissing, &def);
-    delete [] pattern2;
-
-#ifdef HAVE_SETLOCALE
     if (utf8mode)
-        setlocale(LC_CTYPE, orig_locale);
+        setlocale(LC_CTYPE, orig_locale.c_str());
 #endif // HAVE_SETLOCALE
 
+    utf8mode = false;
+    
     return fs;
 }
 
 };
 namespace FbTk {
 
-XmbFontImp::XmbFontImp(const char *filename, bool utf8):m_fontset(0), m_utf8mode(utf8) {
+XmbFontImp::XmbFontImp(const char *filename, bool utf8) : m_fontset(0), m_setextents(0), m_utf8mode(utf8) {
     if (filename != 0)
         load(filename);
 }
@@ -209,7 +167,7 @@ XmbFontImp::~XmbFontImp() {
 }
 
 bool XmbFontImp::load(const std::string &fontname) {
-    if (fontname.size() == 0)
+    if (fontname.empty())
         return false;
 
     XFontSet set = createFontSet(fontname.c_str(), m_utf8mode);
