@@ -29,10 +29,11 @@
 #include "FbTk/App.hh"
 #include "FbTk/Image.hh"
 #include "FbTk/PixmapWithMask.hh"
-#include "FbTk/Surface.hh"
 
 #include "default.xpm"
 
+#include "IconConfig.hh"
+
 #include <X11/xpm.h>
 #include <X11/Xatom.h>
 #include <assert.h>
@@ -52,15 +53,10 @@ namespace FbDesk {
 
 using namespace FbTk;
 
-const FbTk::Font *Icon::s_font = 0;
-GC Icon::s_gc = 0;
 Icon::CacheList Icon::m_cachelist;
-Atom Icon::s_xdnd_atom = 0;
-Icon::TextPlacement Icon::s_textplacement = Icon::TEXTPLACE_BOTTOM;
-int Icon::s_text_alpha = 255;
-int Icon::s_icon_alpha = 255;
 
-Icon::Icon(const std::string &imagefilename, const std::string &command, const std::string &label):
+Icon::Icon(const std::string &imagefilename, 
+           const std::string &command, const std::string &label):
     m_command(command),
     m_image_filename(imagefilename),
     m_label(label),
@@ -79,19 +75,15 @@ Icon::Icon(const std::string &imagefilename, const std::string &command, const s
                    // event mask
                    ButtonPressMask | ButtonReleaseMask | PointerMotionMask | ExposureMask,
                    false), //override redirect
-    m_icon_pixmap(0), m_icon_mask(0) {
-    assert(s_font); // must set this before using this class
-
-    if (s_gc == 0) { 
-        s_gc = DefaultGC(FbTk::App::instance()->display(), m_label_window.screenNumber()); // get gc from screen
-    }
-
+    m_icon_pixmap(0), m_icon_mask(0),
+    m_conf(IconConfig::instance() ) {
+ 
     setupDragAndDrop();
     setupLayer();
 
-    m_icon_window.setAlpha(s_icon_alpha);
-    m_label_window.setAlpha(s_text_alpha);
-
+    m_icon_window.setAlpha(m_conf.iconAlpha());
+    m_label_window.setAlpha(m_conf.textAlpha());
+    m_label_window.setBackgroundColor(m_conf.background());
     loadImage(imagefilename); // load file
     
 }
@@ -100,24 +92,10 @@ Icon::~Icon() {
     removeFromCache(m_image_filename);
 }
 
-void Icon::setTextPlacement(TextPlacement place) { 
-    s_textplacement = place;
-}
-
-void Icon::setTextColor(const FbTk::Color &color) {
-    Display *disp = FbTk::App::instance()->display();
-    if (s_gc == 0)
-        s_gc = DefaultGC(disp, 0);
-    XGCValues gcval;
-    gcval.foreground = color.pixel();
-    XChangeGC(disp, s_gc, GCForeground, &gcval);
-}
-
 void Icon::show() {
     m_icon_window.show();
     m_label_window.show();
     move(m_x, m_y);
-
 }
 
 void Icon::setCommand(const std::string &command) {
@@ -165,8 +143,8 @@ void Icon::resize(unsigned int width, unsigned int height) {
 }
 
 void Icon::update() {
-    m_icon_window.setAlpha(s_icon_alpha);
-    m_label_window.setAlpha(s_text_alpha);
+    m_icon_window.setAlpha(m_conf.iconAlpha());
+    m_label_window.setAlpha(m_conf.textAlpha());
 
     updateLabel();
     updateWindow();
@@ -184,35 +162,36 @@ void Icon::lower() {
 }
 
 void Icon::redrawLabel() {
-    assert(s_font);
     m_label_window.clear();
     m_label_window.updateTransparent();
 
     if (m_label.size() != 0) {
         // setup start position of text
-        int y_start = s_font->ascent();
-        int x_start = 0;
-        if (s_textplacement == TEXTPLACE_LEFT ||
-            s_textplacement == TEXTPLACE_RIGHT) {
+        int y_start = m_conf.font().ascent();
+        int x_start = 2;
+        if (m_conf.textPlacement() == TEXTPLACE_LEFT ||
+            m_conf.textPlacement() == TEXTPLACE_RIGHT) {
             x_start = y_start;
-            y_start = s_font->textWidth(m_label.c_str(), m_label.size());
+            y_start = m_conf.font().textWidth(m_label.c_str(), m_label.size());
         }
 
-        s_font->drawText(m_label_window, m_label_window.screenNumber(), 
-                         s_gc,
-                         m_label.c_str(), m_label.size(),
-                         x_start, y_start);
+        m_conf.font().drawText(m_label_window, m_label_window.screenNumber(), 
+                               m_conf.gc(),
+                               m_label.c_str(), m_label.size(),
+                               x_start, y_start);
     }
 }
 
 void Icon::updateLabel() {
-    if (s_textplacement == TEXTPLACE_LEFT ||
-        s_textplacement == TEXTPLACE_RIGHT) {
-        m_label_window.resize(s_font->height(), 
-                              s_font->textWidth(m_label.c_str(), m_label.size()));
+    if (m_conf.textPlacement() == TEXTPLACE_LEFT ||
+        m_conf.textPlacement() == TEXTPLACE_RIGHT) {
+        // with two extra pixels on each side 
+        m_label_window.resize(m_conf.font().height(), 
+                              m_conf.font().textWidth(m_label.c_str(), m_label.size()) + 4);
     } else {
-        m_label_window.resize(s_font->textWidth(m_label.c_str(), 
-                                                m_label.size()), s_font->height()); 
+        // with two extra pixels on each side 
+        m_label_window.resize(m_conf.font().textWidth(m_label.c_str(), 
+                                                m_label.size()) + 4, m_conf.font().height()); 
     }
     redrawLabel();
 }
@@ -222,14 +201,9 @@ void Icon::updateWindow() {
     if (m_icon_pixmap != 0) {
         m_icon_window.setBackgroundPixmap(m_icon_pixmap);
     }
+    
     m_icon_window.clear();
     m_icon_window.updateTransparent();
-#ifdef HAVE_SHAPE
-    if (m_icon_mask != 0) {
-        XShapeCombineMask(m_icon_window.display(), m_icon_window.window(),
-                          ShapeBounding, 0, 0, m_icon_mask, ShapeSet);
-    }
-#endif // HAVE_SHAPE
     
 }
 
@@ -336,6 +310,7 @@ void Icon::removeFromCache(const std::string &filename) {
 }
 
 void Icon::setupDragAndDrop() {
+    /*
     static int xnd_version = 3;
     Display *disp = FbTk::App::instance()->display();
 
@@ -347,6 +322,7 @@ void Icon::setupDragAndDrop() {
                                  32, PropModeReplace,
                                  (unsigned char *)(&xnd_version), 1);
                     
+    */
 }
 
 void Icon::setupLayer() {
@@ -383,24 +359,22 @@ void Icon::setupLayer() {
 }
 
 void Icon::updateTextAlignment() {
-    if (s_font == 0)
-        return;
 
-    int textwidth = s_font->textWidth(m_label.c_str(), m_label.size());
+    int textwidth = m_conf.font().textWidth(m_label.c_str(), m_label.size());
     if (m_label.size() == 0 || textwidth == 0)
         return;
 
-    switch (s_textplacement) {
+    switch (m_conf.textPlacement()) {
     case TEXTPLACE_BOTTOM:
         // center text under icon	
         m_label_window.move(m_x + m_icon_window.width()/2 - textwidth/2, m_y + height());
         break;
     case TEXTPLACE_TOP:
         // center text above icon
-        m_label_window.move(m_x + m_icon_window.width()/2 - textwidth/2, m_y - s_font->height());
+        m_label_window.move(m_x + m_icon_window.width()/2 - textwidth/2, m_y - m_conf.font().height());
         break;
     case TEXTPLACE_LEFT:
-        m_label_window.move(m_x - s_font->height(), m_y + m_icon_window.height()/2 - textwidth/2);
+        m_label_window.move(m_x - m_conf.font().height(), m_y + m_icon_window.height()/2 - textwidth/2);
         break;
     case TEXTPLACE_RIGHT:
         m_label_window.move(m_x + width(), m_y + m_icon_window.height()/2 - textwidth/2);
