@@ -51,7 +51,7 @@ InputField::InputField(int screen_num, const FbTk::Font &font, GC gc, const FbTk
     setBackgroundColor(bgcolor);
     setGC(gc);
     updateSize();
-    setNoResize(*this);
+    // setNoResize(*this);
 
 
     Atom wm_delete_window = XInternAtom(display(),"WM_DELETE_WINDOW", false);
