@@ -1,5 +1,5 @@
 // main.cc for fbdesk
-// Copyright (c) 2002 - 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at users.sourceforge.net)
 // 
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -62,7 +62,7 @@ void showUsage(const char *name) {
         "-na                         Disable antialias"<<endl<<endl;
 }
 
-int main(int argc, char **argv) {
+int main(int argc, char **argv) try {
     string displaystr, resource_filename("~/.fluxbox/fbdesk");
     bool antialias = true;
     // determine arguments
@@ -73,44 +73,46 @@ int main(int argc, char **argv) {
         } else if (strcmp("-rc", argv[i]) == 0 && // resource file
                    i + 1 < argc) {
             resource_filename = argv[++i];
-        } else if (strcmp("-na", argv[i]) == 0) { // no antialias
-            antialias = false; 
-        } else if (strcmp("-v", argv[i]) == 0) {
-            cout<<"FbDesk "<<FBDESK_VERSION<<" Copyright (c) 2003-2004 Henrik Kinnunen"<<endl;
+        } if (strcmp("-v", argv[i]) == 0) {
+            cout<<"FbDesk "<<FBDESK_VERSION<<" Copyright (c) 2003-2006 Henrik Kinnunen"<<endl;
             exit(0);
         } else {
-            cout<<"FbDesk "<<FBDESK_VERSION<<" Copyright (c) 2003 Henrik Kinnunen"<<endl;
+            cout<<"FbDesk "<<FBDESK_VERSION<<" Copyright (c) 2003-2006 Henrik Kinnunen"<<endl;
             showUsage(argv[0]);
             exit(0);
         }
     }
 
-    try {
+
 		
-        FbTk::App app(displaystr.c_str());
-        // setup signals
-        signal(SIGSEGV, signalhandler);
-        signal(SIGFPE, signalhandler);
-        signal(SIGTERM, signalhandler);
-        signal(SIGINT, signalhandler);
-        signal(SIGUSR1, signalhandler);
-        signal(SIGUSR2, signalhandler);
-        signal(SIGHUP, signalhandler);
-        signal(SIGCHLD, signalhandler);
-        // catch x errors
-        XSetErrorHandler(handleXError);
-	// create fbdesk
-        FbDesk::FbDesk desk(resource_filename.c_str());
-        if (!antialias) {
-            desk.setAntialias(false);
-        }
-        // finaly enter main event loop
-        app.eventLoop();
-    } catch (string error_str) {
-        cerr<<"FbDesk > Error: "<<error_str<<endl;
-    } catch (out_of_range &oor) {
-        cerr<<"Out of range: "<<oor.what()<<endl;
-    } catch (exception &ee) {
-        cerr<<"Exception thrown: "<<ee.what()<<endl;
-    }
-}
+    FbTk::App app(displaystr.c_str());
+    // setup signals
+    signal(SIGSEGV, signalhandler);
+    signal(SIGFPE, signalhandler);
+    signal(SIGTERM, signalhandler);
+    signal(SIGINT, signalhandler);
+    signal(SIGUSR1, signalhandler);
+    signal(SIGUSR2, signalhandler);
+    signal(SIGHUP, signalhandler);
+    signal(SIGCHLD, signalhandler);
+    // catch x errors
+    XSetErrorHandler(handleXError);
+    // create fbdesk
+    FbDesk::FbDesk desk(resource_filename.c_str());
+
+    // finaly enter main event loop
+    app.eventLoop();
+
+    return 0;
+
+ } catch (std::string error_str) {
+    cerr<<"FbDesk > Error: "<<error_str<<endl;
+    return -1;
+ } catch (out_of_range &oor) {
+    cerr<<"Out of range: "<<oor.what()<<endl;
+    return -1;
+ } catch (exception &ee) {
+    cerr<<"Exception thrown: "<<ee.what()<<endl;
+    return -1;
+ }
+
